/* $Header: /fridge/cvs/xscorch/libj/libj.h.in,v 1.12 2009-02-24 05:14:21 jacob Exp $ */
/* 

   libj - libj.h                 Copyright (C) 1998-2003 Justin David Smith
   justins (a) chaos2.org        http://chaos2.org/
    
   Definitions for libj
   

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation, version 2 of the License ONLY.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program, in the file COPYING. If not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA
   
   
   This file is part of LIBJ.
   
*/


/* For xscorch, libj version 4.1.3 */


#ifndef __libj_included
#define __libj_included


/* Include files */
#include <stdlib.h>


/* Setup libj embedded/standalone and other configuration */
#define  LIBJ_STANDALONE      0
#define  LIBJ_C99_STANDARD    1


/* Version information */
#if LIBJ_STANDALONE
#define  __libj_version       ((((long)4 & 0xff) << 24) | (((long)1 & 0xff) << 16) | (((long)3 & 0xff) << 8)) /* Version 4.1.3 */
#endif /* LIBJ_STANDALONE */


/* Compilation definitions */
#define  LIBJ_LIBC_STRING     1

#ifndef TIME_WITH_SYS_TIME
#define  TIME_WITH_SYS_TIME   1
#endif /* TIME_WITH_SYS_TIME */

#ifndef HAVE_SYS_TIME_H
#define  HAVE_SYS_TIME_H      1
#endif /* HAVE_SYS_TIME_H */

#ifndef HAVE_GETTIMEOFDAY
#define  HAVE_GETTIMEOFDAY    1
#endif /* HAVE_GETTIMEOFDAY */


/* Miscellaneous constants */
#define  LIBJ_RK_PRIME     32749 /* Rabin-karp prime number (should be 15-bits max) */
#define  LIBJ_READBUF      0x4000/* Default read buffer size */


/* Data types */
#ifndef LIBJ_TYPES
#define  LIBJ_TYPES              1
typedef  unsigned char     ubyte;
typedef  unsigned short     uword;
typedef  unsigned int    udword;
typedef  unsigned long long    uqword;

typedef  signed   char     sbyte;
typedef  signed   short     sword;
typedef  signed   int    sdword;
typedef  signed   long long    sqword;

typedef  unsigned char     byte;
typedef  unsigned short     word;
typedef  unsigned int    dword;
typedef  unsigned long long    qword;
typedef  unsigned int    sizea;
typedef  signed   int    indexa;
typedef  unsigned short     wchar;
typedef  signed   int    dimensiona;

#define  DWORD_MAX               0xffffffff
#define  WORD_MAX                0xffff
#define  BYTE_MAX                0xff
#endif /* LIBJ_TYPES */


/* Other useful definitions */
#ifndef LIBJ_BOOL
#define  LIBJ_BOOL               1
#define  false                   0
#define  true                    (!false)
#ifndef __cplusplus
typedef  ubyte                   bool;
#endif /* not C++ */
#endif /* LIBJ_BOOL */


/* Useful in manipulating ranges of numbers */
#define  clamp(x, a, b)          ((x) < (a) ? (a) : (x) > (b) ? (b) : (x))
#define  min(x, a)               ((x) < (a) ? (x) : (a))
#define  max(x, a)               ((x) > (a) ? (x) : (a))


/* Typical function defines */
typedef  void (*destructorfn)(void *data);
typedef  bool (*comparefn)(const void *a, const void *b);
typedef  bool (*iteratorfn)(void *a, void *params);


/* Version method */
#if LIBJ_STANDALONE
unsigned int libj_version(void);
#endif /* LIBJ_STANDALONE */


/* __libj_unused is a special attribute indicating a function argument is
   ignored within the function; this is useful for functions which MUST accept
   an argument because they escape, but do not actually access the argument. */
#ifndef __libj_unused
#if __GNUC__ && LIBJ_C99_STANDARD
#define  __libj_unused                  __attribute__((unused))
#else /* Not C99 code */
#define  __libj_unused
#endif /* LIBJ_C99_STANDARD? */  
#endif /* __libj_unused undefined? */


#endif /* Header included? */
